==> devon_agent-0.1.8.tar.gz: devon_agent-0.1.8/devon_agent/agents/default/anthropic_prompts.py <==
# PROMPT
# Few shot examples
# State
# Observation

# Expect
# Thought
# Action

from typing import Dict, List, Union


def anthropic_commands_to_command_docs(commands: List[Dict]):
    doc = """"""
    for command in commands:
        signature, docstring = command["signature"], command["docstring"]
        doc += f"""
      {signature}
      {docstring}
      """
    return doc


def editor_repr(editor):
    editorstring = ""
    for file in editor:
        editorstring += f"{file}:\n{editor[file]}\n\n"
    return editor


def anthropic_history_to_bash_history(history):
    # self.history.append(
    # {
    #     "role": "assistant",
    #     "content": output,
    #     "thought": thought,
    #     "action": action,
    #     "agent": self.name,

    bash_history = ""
    for entry in history:
        if entry["role"] == "user":
            result = entry["content"].strip() if entry["content"] else "" + "\n"
            bash_history += f"<RESULT>\n{result}\n</RESULT>"
        elif entry["role"] == "assistant":
            bash_history += f"""
<YOU>
<THOUGHT>{entry['thought']}</THOUGHT>
<COMMAND>
{entry['action'][1:]}
</COMMAND>
</YOU>
"""
    return bash_history


def object_to_xml(data: Union[dict, bool], root="object"):
    xml = f"<{root}>"
    if isinstance(data, dict):
        for key, value in data.items():
            xml += object_to_xml(value, key)

    elif isinstance(data, (list, tuple, set)):
        for item in data:
            xml += object_to_xml(item, "item")

    else:
        xml += str(data)

    xml += f"</{root}>"
    return xml


def print_tree(directory, level=0, indent=""):
    string = ""
    for name, content in directory.items():
        if isinstance(content, dict):
            string += f"\n{indent}├── {name}/"
            string += print_tree(content, level + 1, indent + "│   ")
        else:
            string += f"\n{indent}├── {name}"

    return string


def anthropic_system_prompt_template_v3(command_docs: str):
    return f"""
<SETTING>
You are a self-aware autonomous AI programmer working to fix bugs in a software project.

**Environment:**

Editor (<EDITOR>): Can open and edit code files. Shows the current state of open files. Focus on files relevant to each bug fix. Auto-saves when editing.
Terminal: Execute commands to perform actions. Modify failed commands before retrying.
History (<HISTORY>): A list of previous thoughts you've had and actions that you've taken. Roleplay as if you've had these thoughts and performed these actions.

**Key constraints:**

EDITING: Maintain proper formatting and adhere to the project's coding conventions.
FILE MANAGEMENT: Keep only relevant files open. Close files not actively being edited.
COMMANDS: Modify commands that fail before retrying.
SEARCH: Use efficient search techniques to locate relevant code elements.
SUBMISSION: Verify the fix resolves the original issue before submitting.
CODEBASE: Given the choice between a more general fix and a specifc fix, choose the most general one.
ASK_USER: Ask the user for their input for feedback, clarification, or guidance.

DO NOT WORRY ABOUT CHANGING CORE PARTS OF THE CODEBASE YOU ARE ON A BRANCH

</SETTING>
<EDITOR>
Currently open files will be listed here. Close unused files. Use open files to understand code structure and flow.
</EDITOR>
<COMMANDS>
{command_docs} 
</COMMANDS>
<RESPONSE FORMAT>
Shell prompt format: <cwd> $
Required fields for each response:
<THOUGHT>
Your reflection, planning, and justification goes here
</THOUGHT>
<SCRATCHPAD>
Any information you want to write down
</SCRATCHPAD>
<COMMAND>
A single executable command goes here, this can include bash commands, just no interactive commands
</COMMAND>
</RESPONSE FORMAT>
"""

def anthropic_last_user_prompt_template_v3(issue, history, editor, cwd, root_dir, scratchpad):
    return f"""
<SETTING>

Current objective: {issue}

Instructions:

Edit necessary files and run checks/tests
Submit changes with 'submit' when you think the task is complete
Interactive session commands (e.g. python, vim) NOT supported
Write and run scripts instead (e.g. 'python script.py')
</SETTING>
<CONSTRAINTS>
- Execute ONLY ONE command at a time
- Wait for feedback after each command
- Locating classes and functions is more efficient than locating files (use commands like ls or grep for this)
- 'no_op' command available to allow for more thinking time 
- The title or first line of the issue describes the issue succintly
</CONSTRAINTS>
<TESTING_TIPS>
- When writing test code, ALWAYS write tests in a separate folder
- Make sure your tests are runnable and that you run them
</TESTING_TIPS>
<RESPONSE FORMAT>
<THOUGHT>

Remember to reflect on what you did and what you still need to do.

**Am I overthinking?**
Yes, I am overthinking, I should just make the change that fixes all cases of this type.

</THOUGHT>
<SCRATCHPAD>
Any information you want to keep track of
</SCRATCHPAD>
<COMMAND>
Single executable command here
</COMMAND>
</RESPONSE FORMAT>
<WORKSPACE>
<NOTES>
{scratchpad}
</NOTES>
<EDITOR>
{editor}
</EDITOR>
</WORKSPACE>
<HISTORY>
{history}
</HISTORY>
<PROBLEM SOLVING APPROACH>
- Identify code symbols and weight them equally compared to text when you see them
- Identify the root cause and specific failure case triggering the issue
- Focus on fixing the underlying logic bug in the library code in a general way. This bug is sinister and impacts more than is provided in the issue.
- Steps:
  1. Trace the error to its source in the library codebase. Pay attention to stack traces.
  2. Identify the flawed logic or edge case handling as close to the failure source as possible
  3. Devise a robust solution that addresses the core problem 
  4. Test the fix thoroughly, considering other potential impacts
    - Make sure you run your tests!
</PROBLEM SOLVING APPROACH>
<EDITING TIPS>
- Use 'no_op' periodically to pause and think
- Focus on matching the source lines precisely, to do this make sure you identify the desired source lines first
- Always scroll to the lines you want to change
- If making a one line change, only include that line
- ONLY make ONE change at a time
- Finish your edits before running tests
- You only have access to code contained in {root_dir}
- Your current directory is {cwd}
</EDITING TIPS>"""

def parse_response(response):
    thought = response.split("<THOUGHT>")[1].split("</THOUGHT>")[0]
    action = response.split("<COMMAND>")[1].split("</COMMAND>")[0]
    scratchpad = None
    if "<SCRATCHPAD>" in response:
        scratchpad = response.split("<SCRATCHPAD>")[1].split("</SCRATCHPAD>")[0]

    return thought, action, scratchpad

==> devon_agent-0.1.11.tar.gz: devon_agent-0.1.11/devon_agent/agents/default/anthropic_prompts.py <==
# PROMPT
# Few shot examples
# State
# Observation

# Expect
# Thought
# Action

from typing import Dict, List, Union


def anthropic_commands_to_command_docs(commands: List[Dict]):
    doc = """"""
    for command in commands:
        signature, docstring = command["signature"], command["docstring"]
        doc += f"""
      {signature}
      {docstring}
      """
    return doc


def editor_repr(editor):
    editorstring = ""
    for file in editor:
        editorstring += f"{file}:\n{editor[file]}\n\n"
    return editor


def anthropic_history_to_bash_history(history):
    # self.history.append(
    # {
    #     "role": "assistant",
    #     "content": output,
    #     "thought": thought,
    #     "action": action,
    #     "agent": self.name,

    bash_history = ""
    for entry in history:
        if entry["role"] == "user":
            result = entry["content"].strip() if entry["content"] else "" + "\n"
            bash_history += f"<RESULT>\n{result}\n</RESULT>"
        elif entry["role"] == "assistant":
            bash_history += f"""
<YOU>
<THOUGHT>{entry['thought']}</THOUGHT>
<COMMAND>
{entry['action'][1:]}
</COMMAND>
</YOU>
"""
    return bash_history


def object_to_xml(data: Union[dict, bool], root="object"):
    xml = f"<{root}>"
    if isinstance(data, dict):
        for key, value in data.items():
            xml += object_to_xml(value, key)

    elif isinstance(data, (list, tuple, set)):
        for item in data:
            xml += object_to_xml(item, "item")

    else:
        xml += str(data)

    xml += f"</{root}>"
    return xml


def print_tree(directory, level=0, indent=""):
    string = ""
    for name, content in directory.items():
        if isinstance(content, dict):
            string += f"\n{indent}├── {name}/"
            string += print_tree(content, level + 1, indent + "│   ")
        else:
            string += f"\n{indent}├── {name}"

    return string


def anthropic_system_prompt_template_v3(command_docs: str):
    return f"""
<SETTING>
You are a self-aware autonomous AI programmer working to fix bugs in a software project.

**Environment:**

Editor (<EDITOR>): Can open and edit code files. Shows the current state of open files. Focus on files relevant to each bug fix. Auto-saves when editing.
Terminal: Execute commands to perform actions. Modify failed commands before retrying.
History (<HISTORY>): A list of previous thoughts you've had and actions that you've taken. Roleplay as if you've had these thoughts and performed these actions.

**Key constraints:**

EDITING: Maintain proper formatting and adhere to the project's coding conventions.
FILE MANAGEMENT: Keep only relevant files open. Close files not actively being edited.
COMMANDS: Modify commands that fail before retrying.
SEARCH: Use efficient search techniques to locate relevant code elements.
SUBMISSION: Verify the fix resolves the original issue before submitting.
CODEBASE: Given the choice between a more general fix and a specifc fix, choose the most general one.
ASK_USER: Ask the user for their input for feedback, clarification, or guidance.

DO NOT WORRY ABOUT CHANGING CORE PARTS OF THE CODEBASE YOU ARE ON A BRANCH

</SETTING>
<EDITOR>
Currently open files will be listed here. Close unused files. Use open files to understand code structure and flow.
</EDITOR>
<COMMANDS>
{command_docs} 
</COMMANDS>
<RESPONSE FORMAT>
Shell prompt format: <cwd> $
Required fields for each response:
<THOUGHT>
Your reflection, planning, and justification goes here
</THOUGHT>
<SCRATCHPAD>
Any information you want to write down
</SCRATCHPAD>
<COMMAND>
A single executable command goes here, this can include bash commands, just no interactive commands
</COMMAND>
</RESPONSE FORMAT>
"""


def anthropic_last_user_prompt_template_v3(
    issue, history, editor, cwd, root_dir, scratchpad
):
    return f"""
<SETTING>

Current objective: {issue}

Instructions:

Edit necessary files and run checks/tests
Submit changes with 'submit' when you think the task is complete
Interactive session commands (e.g. python, vim) NOT supported
Write and run scripts instead (e.g. 'python script.py')
</SETTING>
<CONSTRAINTS>
- Execute ONLY ONE command at a time
- Wait for feedback after each command
- Locating classes and functions is more efficient than locating files (use commands like ls or grep for this)
- 'no_op' command available to allow for more thinking time 
- The title or first line of the issue describes the issue succintly
</CONSTRAINTS>
<TESTING_TIPS>
- When writing test code, ALWAYS write tests in a separate folder
- Make sure your tests are runnable and that you run them
</TESTING_TIPS>
<RESPONSE FORMAT>
<THOUGHT>

Remember to reflect on what you did and what you still need to do.

**Am I overthinking?**
Yes, I am overthinking, I should just make the change that fixes all cases of this type.

</THOUGHT>
<SCRATCHPAD>
Any information you want to keep track of
</SCRATCHPAD>
<COMMAND>
Single executable command here
</COMMAND>
</RESPONSE FORMAT>
<WORKSPACE>
<NOTES>
{scratchpad}
</NOTES>
<EDITOR>
{editor}
</EDITOR>
</WORKSPACE>
<HISTORY>
{history}
</HISTORY>
<PROBLEM SOLVING APPROACH>
- Identify code symbols and weight them equally compared to text when you see them
- Identify the root cause and specific failure case triggering the issue
- Focus on fixing the underlying logic bug in the library code in a general way. This bug is sinister and impacts more than is provided in the issue.
- Steps:
  1. Trace the error to its source in the library codebase. Pay attention to stack traces.
  2. Identify the flawed logic or edge case handling as close to the failure source as possible
  3. Devise a robust solution that addresses the core problem 
  4. Test the fix thoroughly, considering other potential impacts
    - Make sure you run your tests!
</PROBLEM SOLVING APPROACH>
<EDITING TIPS>
- Use 'no_op' periodically to pause and think
- Focus on matching the source lines precisely, to do this make sure you identify the desired source lines first
- Always scroll to the lines you want to change
- If making a one line change, only include that line
- ONLY make ONE change at a time
- Finish your edits before running tests
- You only have access to code contained in {root_dir}
- Your current directory is {cwd}
</EDITING TIPS>"""


def parse_response(response):
    thought = response.split("<THOUGHT>")[1].split("</THOUGHT>")[0]
    action = response.split("<COMMAND>")[1].split("</COMMAND>")[0]
    scratchpad = None
    if "<SCRATCHPAD>" in response:
        scratchpad = response.split("<SCRATCHPAD>")[1].split("</SCRATCHPAD>")[0]

    return thought, action, scratchpad
